#include "File.h"


//Creates copy of map for deleting and replacing records
void File::createMap(std::map<int, Record>& map)
{
	std::ifstream file(filename, std::ios::binary);
	Record curRec;
	int ind = 1; //index of record

	if (!file.is_open())
	{
		throw MyException("Error! Failure in opening output file.");
	}

	while (file.read(reinterpret_cast<char*>(&curRec), sizeof(Record)))
	{
		map.emplace(ind++, curRec);
	}
	file.close();
}


//Adding records into file
void File::addData()
{
	char continue_key = 'y';
	std::ofstream database(filename, std::ios::binary | std::ios::app);

	if (!database.is_open())
		throw MyException("Error! Failure in opening output file.");

	std::cout << "Input records you want to be added into database:\n";

	Record newRec;

	while (continue_key == 'y') //while user wants to add one more record
	{	
		std::cout << "Input new record:\n";
		std::cin >> newRec;
		std::cout << "\n\n";
		database.write(reinterpret_cast<const char*>(&newRec), sizeof(Record));
		std::cout << "Do you want to add one more record into database? [y/n]\n";
		std::cin >> continue_key;
	
		system("cls");
	}
}


//Deleting records from file
void File::delRec()
{
	std::map<int, Record> new_map;
	int ind; //index of record
	char continue_key = 'y';


	createMap(new_map);
	
	while (continue_key == 'y') //while user wants to delete one more record
	{
		std::cout << "\nInput the index of record to be deleted(possible indexes: [1," << new_map.size() << "]):" << std::endl;
		std::cout << "->"; std::cin >> ind;

		if (new_map.erase(ind) == 0) //if index is absents in map
			std::cout << "Wrong index!" << std::endl;
		
		std::cout << "Do you want to delete one more record from file? [y/n]\n";
		std::cin >> continue_key;
		system("cls");
	}

	std::ofstream file(filename, std::ios::binary | std::ios::trunc);

	if (!file.is_open())
	{
		throw MyException("Error! Failure in opening output file.");
	}

	for (const auto& curEl : new_map)
	{
		file.write(reinterpret_cast<const char*>(&curEl.second), sizeof(Record));
	}
}


//Replacing records in file
void File::replaceRec()
{
	std::map<int, Record> new_map;
	Record curRec;
	int ind; //index of record
	char continue_key = 'y';

	
	createMap(new_map);

	while (continue_key == 'y') //while user wants to replace one more record
	{
		std::cout << "\nInput the index of record to be replaced: " << std::endl;
		std::cin >> ind;
		std::cout << "\nInput new record:\n";
		std::cin >> curRec;
		new_map[ind] = curRec;
		std::cout << "Do you want to replace one more record in file? [y/n]\n";
		std::cin >> continue_key;
	}

	std::ofstream file(filename, std::ios::binary | std::ios::trunc);

	if (!file.is_open())
	{
		throw MyException("Error! Failure in opening output file.");
	}

	for (const auto& curEl : new_map)
	{
		file.write(reinterpret_cast<const char*>(&curEl.second), sizeof(Record));
	}
}


//Creates text file based on binary
void File::toTxt() {
	std::map<int, Record> copyMap;
	std::string txt_filename = filename + ".txt";
	std::ofstream file(txt_filename, std::ios::trunc);

	if (!file.is_open()) 
		throw MyException("Error! Failure in opening output file.");

	createMap(copyMap);

	file << "Library. Information about books lending.\n";
	file << "+----+---------------+------------+------------+----------------------------+------------+-------------+-------------------+-------------+-------+\n";
	file << "| ID | Ticket number |   Client   | Issue date | Expiration(amount of days) |   Author   |  Book name  | Distribution year | Distributer | Price |\n";
	file << "+----+---------------+------------+------------+----------------------------+------------+-------------+-------------------+-------------+-------+\n";
	
	for (auto& curEl : copyMap)
	{
		file << "|" << std::setw(3) << curEl.first << " " << curEl.second;
		file << "+----+---------------+------------+------------+----------------------------+------------+-------------+-------------------+-------------+-------+\n";
	}
}


//Searches records with given ticket number
std::vector<Record> File::findTicketNum(int tickNum) {
	std::map<int, Record> map;
	std::vector<Record> resVec;

	createMap(map);

	for (const auto& curEl : map) {
		if (curEl.second.ticketNum == tickNum) {
			resVec.push_back(curEl.second);
		}
	}
	return resVec;
}


//Searches records with given author surname
std::vector<Record> File::findAuthor(const std::string& _author)
{
	std::map<int, Record> map;
	std::vector<Record> resVec;

	createMap(map);

	for (const auto& curEl : map) {
		if (curEl.second.author == _author) {
			resVec.push_back(curEl.second);
		}
	}

	return resVec;
}


//Searches records with given distributer name
std::vector<Record> File::findDistributer(const std::string& _distributer)
{
	std::map<int, Record> map;
	std::vector<Record> resVec;

	createMap(map);

	for (const auto& curEl : map) {
		if (curEl.second.distributer == _distributer) {
			resVec.push_back(curEl.second);
		}
	}

	return resVec;
}


//Searches records with expirationed books
std::vector<Record> File::findAfterDateBooks()
{
	const int NORMA = 30; //maximal value of amount of days to keep book
	std::map<int, Record> map;
	std::vector<Record> resVec;

	createMap(map);

	for (const auto& curEl : map) {
		if (curEl.second.dayCnt > NORMA) {
			resVec.push_back(curEl.second);
		}
	}

	return resVec;
}