#pragma once

#include "Library.h"


//Library interface
void workWithLibrary()
{
	int key = 0;
	std::string filename;
	std::vector<Record> searchResVec; //vector contains search results

	std::cout << "Input the filename:\n";
	std::cout << "-> "; std::cin >> filename;

	File database(filename);

	while (key != 9)
	{
		try
		{
			std::cout << "\n\n";
			showLibraryMenu();
			std::cout << "-> "; std::cin >> key;
			system("cls");

			switch (key)
			{
			case 1:
			{
				std::cout << "1. Add data into database." << std::endl;
				database.addData();
				break;
			}
			case 2:
			{
				std::cout << "2. Delete record from database." << std::endl;
				database.delRec();
				break;
			}
			case 3:
			{
				std::cout << "3. Replace record in database." << std::endl;
				database.replaceRec();
				break;
			}
			case 4:
			{			
				int searchTickNum;

				std::cout << "4. Find client records using ticket number." << std::endl;
				
				std::cout << "Input number of ticket:\n";
				std::cin >> searchTickNum;
				searchResVec = database.findTicketNum(searchTickNum);
				
				if (searchResVec.empty())
				{
					std::cout << "Clients with ticket number " << searchTickNum << " were not found\n";
				}
				else
				{
					printHead();
					for (size_t i = 0; i < searchResVec.size(); i++)
					{
						std::cout << searchResVec[i];
						printPartitionLine();
					}
				}

				searchResVec.clear();
				break;
			}
			case 5:
			{
				std::string searchAuthor;

				std::cout << "5. Find records using author surname." << std::endl;
				
				std::cout << "Input author surname" << std::endl;
				std::cin >> searchAuthor;
				searchResVec = database.findAuthor(searchAuthor);
				
				if (searchResVec.empty())
				{
					std::cout << "Records with author surname \"" << searchAuthor << "\" were not found\n";
				}
				else
				{
					printHead();
					for (size_t i = 0; i < searchResVec.size(); i++)
					{
						std::cout << searchResVec[i];
						printPartitionLine();
					}
				}

				searchResVec.clear();
				break;
			}
			case 6:
			{
				std::string searchDistributer;

				std::cout << "6. Find records using distributer name." << std::endl;
				std::cout << "Input distributer name" << std::endl;
				std::cin >> searchDistributer;
				searchResVec = database.findDistributer(searchDistributer);

				if (searchResVec.empty())
				{
					std::cout << "Records with distributer name \"" << searchDistributer << "\" were not found\n";
				}
				else
				{
					printHead();
					for (size_t i = 0; i < searchResVec.size(); i++)
					{
						std::cout << searchResVec[i];
						printPartitionLine();
					}
				}

				searchResVec.clear();
				break;
			}
			case 7:
			{
				std::cout << "7. Find records with delayed books." << std::endl;
				searchResVec = database.findAfterDateBooks();

				if (searchResVec.empty())
				{
					std::cout << "There are not any expirationed books.\n";
				}
				else
				{
					printHead();
					for (size_t i = 0; i < searchResVec.size(); i++)
					{
						std::cout << searchResVec[i];
						printPartitionLine();
					}
				}
				searchResVec.clear();

				break;
			}

			case 8:
			{
				std::cout << "8. Transform binary file in text file." << std::endl;
				database.toTxt();
				std::cout << "Success!" << std::endl;
				break;
			}
			case 9:
			{
				break;
			}
			default:
			{
				std::cout << "\nWrong input!\n";
			}
			}
		}
		catch (MyException exc)
		{
			std::cout << exc.what();
		}
	}
}


//Prints input hints
void showLibraryMenu()
{
	std::cout << std::endl << "Work with library." << std::endl;
	std::cout << "Choose what to do:" << std::endl;
	std::cout << "1. Add data into database." << std::endl;
	std::cout << "2. Delete record from database." << std::endl;
	std::cout << "3. Replace record in database." << std::endl;
	std::cout << "4. Find client records using ticket number." << std::endl;
	std::cout << "5. Find records using author surname." << std::endl;
	std::cout << "6. Find records using distributer name." << std::endl;
	std::cout << "7. Find records with delayed books." << std::endl;
	std::cout << "8. Transform binary file in text file." << std::endl;
	std::cout << "9. Return to main menu." << std::endl;
}


void printPartitionLine()
{
	std::cout << "+---------------+------------+------------+----------------------------+------------+-------------+-------------------+-------------+-------+\n";
}


void printHead()
{
	std::cout << "+---------------+------------+------------+----------------------------+------------+-------------+-------------------+-------------+-------+\n";
	std::cout << "| Ticket number |   Client   | Issue date | Expiration(amount of days) |   Author   |  Book name  | Distribution year | Distributer | Price |\n";
	std::cout << "+---------------+------------+------------+----------------------------+------------+-------------+-------------------+-------------+-------+\n";
}


//Checks if file is empty
bool emptyFile(const std::string& filename)
{
	std::ifstream file(filename);
	return file.peek() == std::istream::traits_type::eof();
}


//=======================Record methods===================


//Copying constructor
Record::Record(const Record& rec_copy) :
	ticketNum(rec_copy.ticketNum),
	dayCnt(rec_copy.dayCnt),
	distrYear(rec_copy.distrYear),
	price(rec_copy.price)
{
	strcpy_s(surname, rec_copy.surname);
	strcpy_s(issueDate, rec_copy.issueDate);
	strcpy_s(author, rec_copy.author);
	strcpy_s(bookName, rec_copy.bookName);
	strcpy_s(distributer, rec_copy.distributer);
}


//Assignment operator
Record& Record::operator=(const Record& rec_copy) {
	if (this != &rec_copy) {
		ticketNum = rec_copy.ticketNum;
		dayCnt = rec_copy.dayCnt;
		distrYear = rec_copy.distrYear;
		price = rec_copy.price;
		strcpy_s(surname, rec_copy.surname);
		strcpy_s(issueDate, rec_copy.issueDate);
		strcpy_s(author, rec_copy.author);
		strcpy_s(bookName, rec_copy.bookName);
		strcpy_s(distributer, rec_copy.distributer);
	}
	return *this;
}


std::istream& operator>>(std::istream& p_in, Record& rec)
{
	std::string help_str;
	std::string date;


	std::cout << "Input read ticket number -> "; std::cin >> rec.ticketNum;
	
	std::cout << "Input client surname -> "; std::cin >> rec.surname;
	
	std::cout << "Input issue date(d/m/y):";
	std::cout << "\nDay -> "; std::cin >> help_str;
	date = help_str;
	date += '.';
	std::cout << "Month -> "; std::cin >> help_str;
	date += help_str;
	date += '.';
	std::cout << "Year -> "; std::cin >> help_str;
	date += help_str;

	strcpy_s(rec.issueDate, date.c_str());
	
	std::cout << "Input amount of rent days -> "; std::cin >> rec.dayCnt;

	std::cout << "Input author surname -> "; std::cin >> rec.author;

	std::cout << "Input book name -> "; std::cin >> rec.bookName;

	std::cout << "Input distribution year -> "; std::cin >> rec.distrYear;
	
	std::cout << "Input distributer -> "; std::cin >> rec.distributer;

	std::cout << "Input price -> "; std::cin >> rec.price;

	return p_in;
}


std::ostream& operator<<(std::ostream& p_out, const Record& rec) {
	p_out << "|"
		  << std::setw(14) << rec.ticketNum	  << " |"
		  << std::setw(11) << rec.surname	  << " |"
		  << std::setw(11) << rec.issueDate	  << " |"
		  << std::setw(27) << rec.dayCnt	  << " |"
		  << std::setw(11) << rec.author	  << " |"
		  << std::setw(12) << rec.bookName	  << " |"
		  << std::setw(18) << rec.distrYear   << " |"
		  << std::setw(12) << rec.distributer << " |"
		  << std::setw(6)  << rec.price		  << " |"
		  << std::endl;
	
	return p_out;
}


